# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import getpass
import os

from builtins import input
from configparser import ConfigParser


def do_genconfig(args):
    """Generates the config file according to user input

    """
    print("========= Ironic ========= ")
    ironic_auth_url = input("Type the auth_url for the Ironic service: ")
    ironic_username = input("Type your Ironic username: ")
    ironic_tenant = input("Type your Ironic user's tenant name: ")
    ironic_password = getpass.getpass("Type your Ironic user's password: ")
    ironic_insecure = input("Would you like the connections with Ironic to"
                            " be insecure? [y/N]: ") or "N"
    ironic_insecure = 'True' if ironic_insecure.lower() == 'y' else 'False'
    default_deploy_kernel = input("Type in the default deploy keynel image"
                                  " ID on Glance: ")
    default_deploy_ramdisk = input("Type in the default deploy ramdisk "
                                   "image ID on Glance: ")

    # TODO(thiagop): get drivers enabled with ironicclient
    enabled_drivers = ['agent_pxe_oneview', 'iscsi_pxe_oneview']
    ironic_default_driver = input(("Which driver would you like to use? "
                                   "[%s]: ") % ','.join(enabled_drivers))

    print("========= OneView ========= ")
    oneview_manager_url = input("Type in the OneView uri: ")
    oneview_username = input("Type your OneView username: ")
    oneview_password = getpass.getpass("Type your OneView user's password: ")
    allow_insecure = input("Would you like the connections with OneView "
                           "to be insecure? [y/N]: ") or "N"
    allow_insecure = 'True' if allow_insecure.lower() == 'y' else 'False'

    config = ConfigParser()
    config.add_section("ironic")
    config.set("ironic", "auth_url", ironic_auth_url)
    config.set("ironic", "admin_user", ironic_username)
    config.set("ironic", "admin_tenant_name", ironic_tenant)
    config.set("ironic", "admin_password", ironic_password)
    config.set("ironic", "insecure", ironic_insecure)
    config.set("ironic", "default_deploy_kernel_id", default_deploy_kernel)
    config.set("ironic", "default_deploy_ramdisk_id", default_deploy_ramdisk)
    config.set("ironic", "default_driver", ironic_default_driver)
    config.add_section("oneview")
    config.set("oneview", "manager_url", oneview_manager_url)
    config.set("oneview", "username", oneview_username)
    config.set("oneview", "password", oneview_password)
    config.set("oneview", "allow_insecure_connections", allow_insecure)

    filename = input("Type the path to the new configuration file [%s]: "
                     % args.config_file) or args.config_file
    full_filename = os.path.realpath(os.path.expanduser(filename))
    directory = os.path.dirname(full_filename)
    if not os.path.exists(directory):
        os.makedirs(directory)
    with open(full_filename, 'w') as configfile:
        config.write(configfile)
        print("======\nFile created successfully on '%s'!\n======" % filename)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from builtins import input

from ironic_oneviewd.config import ConfClient
from ironic_oneviewd.genconfig.commands import do_genconfig
from ironic_oneviewd.node_manager.manage import NodeManager


def do_manage_ironic_nodes(args):
    """Show a list of OneView servers to be created as nodes in Ironic
    """
    if args.config_file is not "":
        config_file = os.path.realpath(os.path.expanduser(args.config_file))

    defaults = {
        "ca_file": "",
        "insecure": False,
        "tls_cacert_file": "",
        "allow_insecure_connections": False,
    }

    if not os.path.isfile(config_file):
        while True:
            create = input("Config file not found on `%s`. Would you like to "
                           "create one now? [Y/n] " % config_file) or 'y'
            if create.lower() == 'y':
                do_genconfig(args)
                break
            elif create.lower() == 'n':
                return
            else:
                print("Invalid option.\n")

    conf = ConfClient(config_file, defaults)
    node_manager = NodeManager(conf)
    while True:
        node_manager.pull_ironic_nodes()



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

AUTHENTICATION_URL = '/rest/login-sessions'
SERVER_PROFILE_LIST_URI = '/rest/server-profiles?start=0&count=-1'
SERVER_PROFILE_URI = '/rest/server-profiles'
SERVER_HARDWARE_LIST_URI = '/rest/server-hardware?filter="status=Disabled OR'
'status=OK OR status=Warning"'
VOLUMES_URI = '/rest/storage-volumes?start=0&count=-1'

CERTIFICATE_RABBIT_MQ = '/rest/certificates/client/rabbitmq'
CERTIFICATE_AND_KEY_URI = "/rest/certificates/client/rabbitmq/keypair/default"
CA_URI = "/rest/certificates/ca"



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import time

import requests
from requests.packages import urllib3
from oslo_log import log as logging

from ironic_oneviewd import oneview_uri
from ironic_oneviewd.openstack.common._i18n import _LE
from ironic_oneviewd.openstack.common._i18n import _LI
from ironic_oneviewd.openstack.common._i18n import _
from ironic_oneviewd import sync_exceptions as exception


urllib3.disable_warnings()

LOG = logging.getLogger(__name__)

ONEVIEW_POWER_ON = 'On'
ONEVIEW_POWER_OFF = 'Off'


def get_oneview_client(conf):
    kwargs = {
        'username': conf.oneview.username,
        'password': conf.oneview.password,
        'manager_url': conf.oneview.manager_url,
        'allow_insecure_connections': False,
        'tls_cacert_file': ''
    }
    if conf.oneview.allow_insecure_connections.lower() == 'true':
        kwargs['allow_insecure_connections'] = True
    if conf.oneview.tls_cacert_file:
        kwargs['tls_cacert_file'] = conf.oneview.tls_cacert_file
    return OneViewClient(**kwargs)


class OneViewRequestAPI:
    def __init__(self, manager_url, username, password,
                 allow_insecure_connections, tls_cacert_file):
        self.token = None
        self.manager_url = manager_url
        self.username = username
        self.password = password
        self.tls_cacert_file = tls_cacert_file
        self.allow_insecure_connections = allow_insecure_connections
        self.max_retries = 50

    def _get_verify_connection_option(self):
        verify_status = False
        user_cacert = self.tls_cacert_file
        if self.allow_insecure_connections is False:
            if(user_cacert is None):
                verify_status = True
            else:
                verify_status = user_cacert
        return verify_status

    def _is_token_valid(self):
        return self.token is not None

    def _try_execute_request(self, url, request_type, body, headers,
                             verify_status):
        try:
            return requests.request(request_type, url, data=json.dumps(body),
                                    headers=headers, verify=verify_status)
        except requests.RequestException as ex:
            LOG.error(_LE("Can't connect to OneView: %s")
                      % (str(ex.message).split(':')[-1]))
            LOG.error(("Can't connect to OneView: %s")
                      % (str(ex.message).split(':')[-1]))
            raise exception.OneViewConnectionError(
                "Can't connect to OneView: %s" % str(ex.message))

    def _new_token(self):
        LOG.info(
            _LI("Using OneView credentials specified in configuration file")
        )
        url = '%s%s' % (self.manager_url,
                        oneview_uri.AUTHENTICATION_URL)
        body = {
            'password': self.password,
            'userName': self.username
        }
        headers = {'content-type': 'application/json'}
        verify_status = self._get_verify_connection_option()
        if verify_status is False:
            LOG.warn('Using insecure connection')
        json_response = None
        repeat = True
        while repeat:
            r = self._try_execute_request(url, 'POST', body, headers,
                                          verify_status)
            # NOTE: Workaround to fix JsonDecode problems
            try:
                json_response = r.json()
                repeat = self._check_request_status(r)
            except:
                repeat = True
        return json_response.get('sessionID')

    def _update_token(self):
        if not self._is_token_valid():
            self.token = self._new_token()

    def _check_request_status(self, response):
        repeat = False
        status = response.status_code
        response_json = response.json()
        if status in (409,):
            ignored_conflicts = {'RABBITMQ_CLIENTCERT_CONFLICT'}
            if (response_json.get('errorCode') in ignored_conflicts):
                repeat = False
            else:
                time.sleep(10)
                repeat = True
            LOG.debug("Conflict contacting OneView: ", response_json)
        elif status in (404, 500):
            LOG.error(_LE("Error contacting OneView: "), response_json)
            LOG.error(("Error contacting OneView: "), response_json)
        elif status not in (200, 202):
            LOG.warn("Status not recognized:", status, response_json)
        return repeat

    def prepare_and_do_request(self, uri, body={}, request_type='GET',
                               api_version='200'):
        max_retries = self.max_retries

        self._update_token()
        headers = {
            'content-type': 'application/json',
            'X-Api-Version': api_version,
            'Auth': self.token
        }
        url = '%s%s' % (self.manager_url, uri)
        verify_status = self._get_verify_connection_option()

        json_response = None
        repeat = True
        retries = 0
        while repeat and (retries < max_retries):
            r = self._try_execute_request(url, request_type, body, headers,
                                          verify_status)
            # NOTE: Workaround to fix JsonDecode problems
            try:
                json_response = r.json()
                repeat = self._check_request_status(r)
            except Exception:
                repeat = True
                retries += 1
        return json_response


class OneViewServerHardwareAPI(OneViewRequestAPI):

    def get_server_hardware(self, server_hardware_uri):
        server_hardware_json = self.prepare_and_do_request(server_hardware_uri)
        if server_hardware_json.get("uri") is None:
            raise exception.OneViewResourceNotFoundError()

        return server_hardware_json

    def get_server_hardware_list(self):
        uri = "/rest/server-hardware?start=0&count=-1"
        server_hardwares = self.prepare_and_do_request(uri)
        return server_hardwares.get("members")

    def get_server_profile_assigned_to_sh(self, server_hardware_uri):
        server_hardware_json = self.get_server_hardware(server_hardware_uri)
        return server_hardware_json.get('serverProfileUri')

    def parse_server_hardware_to_dict(self, server_hardware):
        port_map = server_hardware.get('portMap')
        try:
            slot = port_map.get('deviceSlots')[0]
            physical_ports = slot.get('physicalPorts')
            mac_address = physical_ports[0].get('mac')
        except Exception:
            raise Exception("Server hardware primary physical NIC not found.")
        vcpus = (server_hardware["processorCoreCount"] *
                 server_hardware["processorCount"])
        return {'name': server_hardware["name"],
                'cpus': vcpus,
                'memory_mb': server_hardware["memoryMb"],
                'local_gb': 120,
                'server_hardware_uri': server_hardware["uri"],
                'server_hardware_type_uri':
                    server_hardware["serverHardwareTypeUri"],
                'enclosure_group_uri': server_hardware['serverGroupUri'],
                'cpu_arch': 'x86_64',
                'mac': mac_address,
                'server_profile_uri': server_hardware.get('serverProfileUri')
                }


class OneViewServerProfileTemplateAPI(OneViewRequestAPI):
    def generate_server_profile_from_server_profile_template(
            self, server_profile_template_uri, server_profile_name,
            server_profile_server_hardware_uri):
        new_server_profile = self.prepare_and_do_request(
            uri=server_profile_template_uri + "/new-profile")
        new_server_profile['name'] = server_profile_name
        new_server_profile['serverHardwareUri'] = (
            server_profile_server_hardware_uri
        )
        return new_server_profile


class OneViewServerProfileAPI(OneViewRequestAPI):
    def list(self):
        return self.prepare_and_do_request(
            uri=oneview_uri.SERVER_PROFILE_LIST_URI).get('members')

    def get(self, server_profile_uri):
        return self.prepare_and_do_request(uri=server_profile_uri)

    def _wait_to_assign(self, task_uri):
        max_retries = self.max_retries
        retries = 0

        task = self.prepare_and_do_request(uri=task_uri)
        while task['taskState'] != 'Completed' and retries < max_retries:
            retries += 1
            time.sleep(5)
            task = self.prepare_and_do_request(uri=task_uri)
        if task['taskState'] != 'Completed':
            raise exception.OneViewMaxRetriesExceededError(
                _("Server profile wasn't applied after %s retries.")
                % max_retries
            )
        return task

    def create(self, server_profile):
        task = self.prepare_and_do_request(
            uri=oneview_uri.SERVER_PROFILE_URI,
            body=server_profile,
            request_type='POST'
        )
        task = self._wait_to_assign(task['uri'])
        return task['associatedResource']['resourceUri']

    def delete(self, server_profile_name):
        uri = '/rest/server-profiles?filter=name="%s"' % (server_profile_name)
        task = self.prepare_and_do_request(uri=uri, request_type='DELETE')
        task = self._wait_to_assign(task['uri'])

    def server_profile_template_list(self):
        return [server_profile for server_profile in self.list()
                if server_profile.get('serverHardwareUri') is None]

    def get_server_profile_template(self, server_profile_template_uri):
        if server_profile_template_uri is None:
            raise exception.OneViewServerProfileTemplateError()

        server_profile_template_json = self.prepare_and_do_request(
            uri=server_profile_template_uri, request_type='GET'
        )

        if server_profile_template_json.get("uri") is None:
            raise exception.OneViewResourceNotFoundError()

        return server_profile_template_json

    def clone_and_assign(self, server_hardware_uri,
                         server_profile_template_uri, node_uuid):
        max_retries = self.max_retries
        server_profile_template = self.get_server_profile_template(
            server_profile_template_uri)
        sh_api = OneViewServerHardwareAPI()
        server_hardware = sh_api.get_server_hardware(server_hardware_uri)
        server_profile_name = "Ironic [%s]" % (node_uuid)
        server_profile_clone_body = self._build_clone_body(
            server_profile_template, server_hardware, server_profile_name)
        ret = self.prepare_and_do_request(
            uri="/rest/server-profiles",
            body=server_profile_clone_body,
            request_type="POST"
        )
        if 'taskStatus' not in ret:
            raise exception.OneViewServerProfileCloneError()
        else:
            isAssigned = False
            retries = 0
            while not isAssigned and retries < max_retries:
                retries += 1
                time.sleep(5)
                server_hardware = sh_api.get_server_hardware(
                    server_hardware_uri)
                isAssigned = server_hardware.get('state') == 'ProfileApplied'
            if not isAssigned:
                raise exception.OneViewMaxRetriesExceededError(
                    _("Server profile wasn't applied after %s retries.")
                    % max_retries
                )
        server_hardware = sh_api.get_server_hardware(server_hardware_uri)
        return server_hardware.get('serverProfileUri')

    def generate_and_assign_server_profile_from_server_profile_template(
            self, server_profile_template_uri, server_profile_name,
            server_profile_server_hardware_uri):
        ov_spt_api = OneViewServerProfileTemplateAPI(
            self.manager_url,
            self.username,
            self.password,
            self.allow_insecure_connections,
            self.tls_cacert_file
        )

        new_server_profile_dict = (
            ov_spt_api.generate_server_profile_from_server_profile_template(
                server_profile_template_uri, server_profile_name,
                server_profile_server_hardware_uri
            )
        )
        return self.create(new_server_profile_dict)

    def unassign_server_profile(self, server_hardware_uri, server_profile_uri):
        max_retries = self.max_retries
        sh_api = OneViewServerHardwareAPI(self.manager_url,
                                          self.username,
                                          self.password,
                                          self.allow_insecure_connections,
                                          self.tls_cacert_file)
        server_profile_body = self.prepare_and_do_request(
            uri=server_profile_uri)
        server_profile_body['serverHardwareUri'] = None
        server_profile_body['enclosureUri'] = None
        server_profile_body['enclosureBay'] = None

        self.prepare_and_do_request(
            uri=server_profile_uri,
            body=server_profile_body,
            request_type="PUT"
        )

        isNotAssigned = False
        retries = 0
        while not isNotAssigned and retries < max_retries:
            retries += 1
            time.sleep(5)
            isNotAssigned = (
                sh_api.get_server_hardware(server_hardware_uri).get('state') ==
                'NoProfileApplied'
            )
        if not isNotAssigned:
            raise exception.OneViewMaxRetriesExceededError(
                _("Server profile wasn't removed successfuly after %s retries")
                % max_retries
            )


class ResourceAPI(OneViewRequestAPI):
    def get(self, uri, field=None):
        resource = self.prepare_and_do_request(uri)
        return resource if field is None else resource[field]

    def _list(self, uri, fields=None):
        obj_list = self.prepare_and_do_request(uri).get("members")
        if fields is None:
            return obj_list

        filtered_list = []

        for obj_dict in obj_list:
            if self._is_dict_elegible(obj_dict, fields):
                filtered_list.append(obj_dict)

        return filtered_list

    def _is_dict_elegible(self, obj_dict, fields):
        for key, value in fields.items():
            if obj_dict[key] != value:
                return False
        return True


class OneViewCertificateAPI(ResourceAPI):
    def get_certificate(self):
        return self.get(oneview_uri.CERTIFICATE_AND_KEY_URI,
                        'base64SSLCertData')

    def get_key(self):
        return self.get(oneview_uri.CERTIFICATE_AND_KEY_URI,
                        'base64SSLKeyData')

    def get_ca(self):
        return self.get(oneview_uri.CA_URI)

    def post_certificate(self):
        body = {'type': 'RabbitMqClientCertV2', 'commonName': 'default'}

        return self.prepare_and_do_request(oneview_uri.CERTIFICATE_RABBIT_MQ,
                                           body=body, request_type='POST')


class OneViewServerHardwareTypeAPI(ResourceAPI):
    pass


class OneViewEnclosureGroupAPI(ResourceAPI):
    pass


class OneViewClient:
    def __init__(self, manager_url, username, password,
                 allow_insecure_connections, tls_cacert_file):
        self.certificate = OneViewCertificateAPI(
            manager_url,
            username,
            password,
            allow_insecure_connections,
            tls_cacert_file
        )
        self.server_hardware = OneViewServerHardwareAPI(
            manager_url,
            username,
            password,
            allow_insecure_connections,
            tls_cacert_file
        )
        self.server_profile = OneViewServerProfileAPI(
            manager_url,
            username,
            password,
            allow_insecure_connections,
            tls_cacert_file
        )
        self.server_hardware_type = OneViewServerHardwareTypeAPI(
            manager_url,
            username,
            password,
            allow_insecure_connections,
            tls_cacert_file
        )
        self.enclosure_group = OneViewEnclosureGroupAPI(
            manager_url,
            username,
            password,
            allow_insecure_connections,
            tls_cacert_file
        )



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironic_oneviewd.openstack.common._i18n import _


class InvalidParameterValue(Exception):
    pass


class MissingParameterValue(Exception):
    pass


class OneViewConnectionError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class OneViewNotAuthorizedException(Exception):
    message = _("Unauthorized access to OneView. Check credentials in"
                " ironic.conf.")


class OneViewResourceNotFoundError(Exception):
    message = _("Resource not Found in OneView")


class OneViewServerProfileTemplateError(Exception):
    message = _("Server Profile Template not found into driver_info")


class OneViewServerProfileCloneError(Exception):
    message = _("Error cloning server profile")


class OneViewMaxRetriesExceededError(Exception):
    message = _("Max connection retries to OneView exceeded")


class OneViewBootDeviceInvalidError(Exception):
    message = _("The device is not valid to setup the boot order")


class OneViewServerProfileAssociatedError(Exception):
    message = _("There is no Server Profile associated to this Server"
                " Hardware")


class OneViewErrorStateSettingPowerState(Exception):
    message = _("Get Error State in OneView trying to set power state of "
                "Server Hardware")



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironic_oneviewd.oneview_client import get_oneview_client
from ironic_oneviewd.openstack_client import get_ironic_client
from ironic_oneviewd.openstack_client import get_nova_client


class Facade(object):

    def __init__(self, config):
        self.ironicclient = get_ironic_client(config)
        self.novaclient = get_nova_client(config)
        self.oneviewclient = get_oneview_client(config)

    # =========================================================================
    # Ironic actions
    # =========================================================================
    def get_ironic_node_list(self):
        return self.ironicclient.node.list(detail=True)

    def get_ironic_node(self, node_uuid):
        return self.ironicclient.node.get(node_uuid)

    def node_set_maintenance(self, node_uuid, maintenance_mode, maint_reason):
        return self.ironicclient.node.set_maintenance(
            node_uuid,
            maintenance_mode,
            maint_reason=maint_reason
        )

    def create_ironic_node(self, **attrs):
        return self.ironicclient.node.create(**attrs)

    def set_node_provision_state(self, node, state):
        return self.ironicclient.node.set_provision_state(node.uuid, state)

    def create_node_port(self, node_uuid, port_mac_address):
        return self.ironicclient.port.create(node_uuid=node_uuid,
                                             address=port_mac_address)

    # =========================================================================
    # OneView actions
    # =========================================================================

    def prepare_and_do_ov_requests(self, uri, body={}, request_type='GET',
                                   api_version='120'):
        return self.oneviewclient.prepare_and_do_request(uri, body,
                                                         request_type,
                                                         api_version)

    def get_server_hardware(self, uri):
        return self.oneviewclient.server_hardware.get_server_hardware(uri)

    def get_server_profile_assigned_to_sh(self, server_hardware_uri):
        return self.oneviewclient.server_hardware.get_server_profile_assigned_to_sh(server_hardware_uri)

    def parse_server_hardware_to_dict(self, server_hardware_json):
        return self.oneviewclient.server_hardware.parse_server_hardware_to_dict(server_hardware_json)

    def get_ov_server_hardware_list(self,):
        return self.oneviewclient.server_hardware.get_server_hardware_list()

    def get_ov_server_power_state(self, driver_info):
        return self.oneviewclient.server_hardware.get_node_power_state(
            driver_info)

    def get_server_profile(self, server_profile_uri):
        return self.oneviewclient.server_profile.get_server_profile_template(
            server_profile_uri)

    def generate_and_assign_server_profile_from_server_profile_template(
        self, server_profile_template_uri, server_profile_name,
        server_profile_server_hardware_uri):
        return self.oneviewclient.server_profile.generate_and_assign_server_profile_from_server_profile_template(
            server_profile_template_uri, server_profile_name,
            server_profile_server_hardware_uri)

    def clone_and_assign_server_profile(self, server_hardware_uri,
                                        server_profile_template_uri,
                                        node_uuid):
        return self.oneviewclient.server_profile.clone_and_assign(
            server_hardware_uri, server_profile_template_uri, node_uuid)

    def unassign_server_profile(self, server_hardware_uri, server_profile_uri):
        return self.oneviewclient.server_profile.unassign_server_profile(server_hardware_uri, server_profile_uri)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient import client as ironic_client
from novaclient import client as nova_client

from ironic_oneviewd.config import ConfClient
from ironic_oneviewd.oneview_client import OneViewServerHardwareAPI
from ironic_oneviewd import service_logging as logging


LOG = logging.getLogger(__name__)

IRONIC_API_VERSION = '1.11'


def get_ironic_client(conf):
    kwargs = {
        'os_username': conf.ironic.admin_user,
        'os_password': conf.ironic.admin_password,
        'os_auth_url': conf.ironic.auth_url,
        'os_tenant_name': conf.ironic.admin_tenant_name,
        'os_ironic_api_version': IRONIC_API_VERSION,
    }
    if conf.ironic.insecure.lower() == 'true':
        kwargs['insecure'] = True
    if conf.ironic.ca_file:
        kwargs['ca_file'] = conf.ironic.ca_file

    LOG.debug("Using OpenStack credentials specified in the configuration file"
              " to get Ironic Client")
    ironicclient = ironic_client.get_client(1, **kwargs)
    return ironicclient


def get_nova_client(conf):
    kwargs = {
        'auth_url': conf.nova.auth_url,
        'username': conf.nova.username,
        'password': conf.nova.password,
        'tenant_name': conf.nova.tenant_name
    }
    if conf.nova.insecure.lower() == 'true':
        kwargs['insecure'] = True
    if conf.nova.ca_file:
        kwargs['ca_file'] = conf.nova.ca_file
    else:
        kwargs['ca_file'] = None
    LOG.debug("Using OpenStack credentials specified in the configuration file"
              " to get Nova Client")
    nova = nova_client.Client(2, conf.nova.username,
                              conf.nova.password,
                              conf.nova.tenant_name,
                              conf.nova.auth_url,
                              kwargs['insecure'],
                              kwargs['ca_file'])
    return nova


class OpenstackClient:
    def __init__(self, configname, **kwargs):
        self.conf_client = ConfClient(configname)
        self.ca_file = kwargs.get('os_cacert')
        self.insecure = kwargs.get('os_insecure', False)

    def _update_ironic_node_state(self, node, server_hardware_uri):
        oneview_sh_client = OneViewServerHardwareAPI()
        state = oneview_sh_client.get_node_power_state(server_hardware_uri)

        LOG.info('Setting node %(node_uuid)s power state to %(state)s',
                 {'node_uuid': node.uuid, 'state': state})

        self._get_ironic_client().node.set_power_state(node.uuid, state)

    def set_ironic_node_provision_state(self, node, target_state):
        LOG.debug('Setting node %(node_uuid)s provision state to %(state)s',
                  {'node_uuid': node.uuid, 'state': target_state})
        ironic_client = self._get_ironic_client()
        ironic_client.node.set_provision_state(node.uuid, target_state)

    def create_ironic_node_port(self, node_uuid, port_mac):
        LOG.debug(
            'Creating port for node %(node_uuid)s', {'node_uuid': node_uuid}
        )
        ironic_client = self._get_ironic_client()
        ironic_client.port.create(node_uuid=node_uuid, address=port_mac)

    def _is_flavor_available(self, server_hardware_info):
        LOG.info("Getting flavors from nova")
        for flavor in self.get_nova_client().flavors.list():
            extra_specs = flavor.get_keys()
            if('capabilities:server_hardware_type_uri' in extra_specs):
                if(extra_specs.get('capabilities:server_hardware_type_uri') !=
                    server_hardware_info.get('server_hardware_type_uri')):
                    continue
                if(extra_specs.get('cpu_arch') !=
                    server_hardware_info.get('cpu_arch')):
                    continue
                if(flavor._info.get('vcpus') !=
                    server_hardware_info.get('cpus')):
                    continue
                if(flavor._info.get('ram') !=
                    server_hardware_info.get('memory_mb')):
                    continue
                return True
        return False

    def flavor_list(self):
        nova_client = self.get_nova_client()
        return nova_client.flavors.list()



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironic_oneviewd.openstack.common._i18n import _
from ironic_oneviewd import sync_exceptions
import oneview_client


REQUIRED_ON_PROPERTIES = {
    'server_hardware_type_uri': _("Server Hardware Type URI Required."),
    'server_profile_template_uri': _("Server Profile Template URI required"),
}

REQUIRED_ON_EXTRAS = {
    'server_hardware_uri': _("Server Hardware URI. Required."),
}


def node_has_server_profile(driver_info):
    server_hardware = oneview_client.get_server_hardware(driver_info)
    server_profile_uri = server_hardware.get("serverProfileUri")
    return server_profile_uri is not None


def verify_node_properties(node):
    properties = node.properties.get('capabilities', '')
    for key in REQUIRED_ON_PROPERTIES:
        if key not in properties:
            raise sync_exceptions.MissingParameterValue(
                _("Missing the following OneView data in node's "
                  "properties/capabilities: %s.") % key
            )

    return properties


def verify_node_extra(node):
    extra = node.extra or {}
    for key in REQUIRED_ON_EXTRAS:
        if not extra.get(key):
            raise sync_exceptions.MissingParameterValue(
                _("Missing the following OneView data in node's extra: %s.")
                % key
            )

    return extra


def capabilities_to_dict(capabilities):
    capabilities_dict = {}
    if capabilities:
        try:
            for capability in capabilities.split(','):
                key, value = capability.split(':')
                capabilities_dict[key] = value
        except ValueError:
            raise sync_exceptions.InvalidParameterValue(
                _("Malformed capabilities value: %s") % capability
            )

    return capabilities_dict



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

from oslo_log import log as logging

from ironic_oneviewd import facade
from ironic_oneviewd import sync_exceptions
from ironic_oneviewd.openstack.common._i18n import _

LOG = logging.getLogger(__name__)

ENROLL_PROVISION_STATE = 'enroll'
MANAGEABLE_PROVISION_STATE = 'manageable'


class NodeManager:
    def __init__(self, conf_client):

        self.supported_drivers = {"agent_pxe_oneview",
                                  "iscsi_pxe_oneview",
                                  "fake_oneview"}

        self.conf_client = conf_client
        self.facade = facade.Facade(conf_client)

    def pull_ironic_nodes(self):
        ironic_nodes = self.facade.get_ironic_node_list()
        for node in ironic_nodes:
            if node.driver in self.supported_drivers:
                try:
                    self.manage_node_provision_state(node)
                except Exception:
                    print('Something went wrong reading node info.')

    def manage_node_provision_state(self, node):
        provision_state = node.provision_state
        if node.maintenance:
            return
        elif provision_state == ENROLL_PROVISION_STATE:
            self.take_enroll_state_actions(node)
        elif provision_state == MANAGEABLE_PROVISION_STATE:
            self.take_manageable_state_actions(node)

    def capabilities_to_dict(self, capabilities):
        """Parse the capabilities string into a dictionary
        :param capabilities: the node capabilities as a formatted string
        :raises: InvalidParameterValue if capabilities is not an string or has
                 a malformed value
        """
        capabilities_dict = {}
        if capabilities:
            if not isinstance(capabilities, six.string_types):
                raise sync_exceptions.InvalidParameterValue(
                    _("Value of 'capabilities' must be string. Got %s")
                    % type(capabilities))
            try:
                for capability in capabilities.split(','):
                    key, value = capability.split(':')
                    capabilities_dict[key] = value
            except ValueError:
                raise sync_exceptions.InvalidParameterValue(
                    _("Malformed capabilities value: %s") % capability
                )

        return capabilities_dict

    def take_enroll_state_actions(self, node):
        # TODO (sinval): temos de validar se node_capabilities existem
        # TODO (sinval): validar se essas coisas sao None
        node_server_hardware_uri = node.driver_info.get('server_hardware_uri')
        node_capabilities = self.capabilities_to_dict(
            node.properties.get('capabilities')
        )
        node_server_profile_template_uri = node_capabilities.get(
            'server_profile_template_uri'
        )
        server_hardware_dict = self.facade.get_server_hardware(
            node_server_hardware_uri
        )
        sh_server_profile_uri = server_hardware_dict.get('serverProfileUri')
        if sh_server_profile_uri is not None:
            LOG.error("The Server Hardware already has a "
                      "Server Profile applied.")
        else:
            self.apply_enroll_node_configuration(
                node_server_hardware_uri,
                node_server_profile_template_uri,
                node.uuid
            )
            self.facade.set_node_provision_state(node, 'manage')

    def apply_enroll_node_configuration(self, server_hardware_uri,
                                        server_profile_template_uri,
                                        node_uuid):
        server_profile_name = "Ironic [%s]" % (node_uuid)
        sp_applied_uri = self.facade.\
            generate_and_assign_server_profile_from_server_profile_template(
                server_profile_template_uri, server_profile_name,
                server_hardware_uri)

        sp_dict = self.facade.get_server_profile(sp_applied_uri)
        server_profile_mac = sp_dict.get('connections')[0].get('mac')
        self.facade.create_node_port(node_uuid, server_profile_mac)
        # TODO(sinval) config volumes (SAN storage)

    def take_manageable_state_actions(self, node):
        self.facade.set_node_provision_state(node, 'provide')



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Universidade Federal de Campina Grande
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

_formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
_loggers = {}


def _getHandler(filename, formatter):
    handler = logging.FileHandler(filename)
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(formatter)

    return handler


def getLogger(name, servicename='service'):
    filename = servicename + '.log'
    if name not in _loggers:
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)

        logger.addHandler(_getHandler(filename=filename,
                                      formatter=_formatter))
        _loggers[name] = logger
    return _loggers[name]
